@@ -1,6 +1,4 @@
-"""
-IRC string comparison utilities
-"""
+"""IRC string comparison utilities."""
 
 import re
 from typing import Final
@@ -11,9 +9,7 @@ GLOB_MAP: Final = {"?": ".", "*": ".*"}
 
 
 def match_mask(mask: str, pattern: str) -> bool:
-    """
-    Match hostmask patterns in the standard banmask syntax (eg '*!*@host')
-    """
+    """Match hostmask patterns in the standard banmask syntax (eg '*!*@host')."""
     re_pattern = ""
     for c in pattern:
         re_pattern += GLOB_MAP.get(c, re.escape(c))
